#ifndef __XLIC_LICENSING__
#define __XLIC_LICENSING__

#ifndef XLICD_SOCKETFILE
#define XLICD_SOCKETFILE "/run/xlicd/xlicd.sock"
#endif

#ifdef __cplusplus

#include <cstddef>
#include <cstdint>
#include <ctime>

extern "C" {

#else // __cplusplus

#include <stddef.h>
#include <stdint.h>
#include <time.h>

#endif // __cplusplus

typedef enum {
	XLIC_STATE_UNKNOWN = 0,
	XLIC_STATE_INACTIVE = 1,
	XLIC_STATE_ACTIVE = 2,
	XLIC_STATE_GRACE = 3,
	XLIC_STATE_INVALID = 4,
	XLIC_STATE_FOREIGN = 6,
} xlic_license_state;

typedef enum {
	XLIC_TYPE_UNKNOWN = 0,
	XLIC_TYPE_PHYSICAL = 1,
	XLIC_TYPE_VIRTUAL = 2,
} xlic_license_type;

typedef enum {
	XLIC_RESULT_SUCCESS = 0,
	XLIC_RESULT_GENERIC_ERROR = -1, // general error
	XLIC_RESULT_SERVER_ERROR = -2, // server communication error
	XLIC_RESULT_DAEMON_ERROR = -3, // daemon communication error
	XLIC_RESULT_REJECTED = -4, // request rejected by server
	XLIC_RESULT_INVALIDKEY = -5, // invalid license key
	XLIC_RESULT_NOLICENSE = -6, // license does not exists
	XLIC_RESULT_KEYEXISTS = -7, // license already exists
	XLIC_RESULT_UNKNOWN = -8,
	XLIC_RESULT_FOREIGN = -9, // foreign licenses
	XLIC_RESULT_INVALIDNAME = -10, // invalid license name
	XLIC_RESULT_INVALIDUSERAUTH = -11, // invalid username/password pair
	XLIC_RESULT_PARSE_ERROR = -12, // json parse error
	XLIC_RESULT_INVALID_SYSTEM_NAME = -13, // invalid system name
	XLIC_RESULT_INVALID_APPKEY = -14, // invalid application key
	XLIC_RESULT_INVALID_METRIC_NAME = -15, // invalid metric name
	XLIC_RESULT_NODONGLE = -16, // dongle required but not present
	XLIC_RESULT_INVALID_PRODUCT_NAME = -17, // invalid product name
	XLIC_RESULT_INVALID_REQUEST = -18, // server reported invalid request
	XLIC_RESULT_SERIAL_NUMBER_REQUIRED = -19, // server reported missing serial number
	XLIC_RESULT_SERIAL_NUMBER_INVALID = -20, // server reported invalid serial number
	XLIC_RESULT_SERIAL_NUMBER_EXISTS = -21, // server reported serial number already taken
	XLIC_RESULT_SYSTEM_NAME_REQUIRED = -22, // server reported missing system name
	XLIC_RESULT_SYSTEM_NAME_INVALID = -23, // server reported invalid system name
	XLIC_RESULT_SYSTEM_NAME_EXISTS = -24, // server reported system name already taken
	XLIC_RESULT_MAC_ADDRESS_REQUIRED = -25, // server reported missing MAC address
	XLIC_RESULT_MAC_ADDRESS_INVALID = -26, // server reported invalid MAC address
	XLIC_RESULT_MAC_ADDRESS_EXISTS = -27, // server reported MAC address already taken
	XLIC_RESULT_COMPANY_REQUIRED = -28, // server reported missing company
	XLIC_RESULT_COMPANY_INVALID = -29, // server reported invalid company
	XLIC_RESULT_COMPANY_EXISTS = -30, // server reported company already taken
} xlic_result;

struct xlic_license_list {
	size_t count;
	struct xlic_license * licenses;
};

struct xlic_license {
	xlic_license_state state;
	xlic_license_type type;
	const char * key;
	const char * name;
	const char * description;
	time_t valid_after;
	size_t valid_for;
	size_t grace_days;
	size_t attribute_count;
	struct xlic_license_attribute * attributes;
};

struct xlic_license_attribute {
	const char * name;
	const char * value;
};

extern int xlic_fetch(const char * socket_filename);
extern int xlic_licenses(const char * socket_filename, struct xlic_license_list **);
extern void xlic_free(void * ptr);
extern int xlic_register_license(const char * socket_filename, const char * license_key);
extern int xlic_request_license(const char * socket_filename, const char * license_name,
	const char * username, const char * password);
extern int xlic_unregister_license(const char * socket_filename, const char * license_key);
extern int xlic_unregister_product(const char * socket_filename, const char * product_name);
extern const char * xlic_errstr(int code);
extern int xlic_update(const char * socket_filename);
extern int xlic_clear(const char * socket_filename);
extern int xlic_set_system_name(const char * socket_filename, const char * system_name);
extern int xlic_system_name(const char * socket_filename, char ** system_name);
extern int xlic_set_metric(const char * socket_filename, const char * application_key,
	const char * metric_name, const int value);
extern int xlic_metric(const char * socket_filename, const char * metric_name, int * value);

#ifdef __cplusplus
}
#endif // __cplusplus

#endif // __XLIC_LICENSING__
